@@ -9,5 +9,5 @@
 
 . ./include.sh
 
-${examples_dir}f_set_pv > /dev/null
+${examples_dir}grib_api_f_set_pv > /dev/null
 rm -f out.pv.grib1
