@@ -161,7 +161,7 @@ static void init_class(grib_accessor_class* c)
 
 static int first = 1;
 
-#define JASPER_LIB 1
+#define JASPER_LIB   1
 #define OPENJPEG_LIB 2
 
 static void init(grib_accessor* a,const long v, grib_arguments* args)
@@ -169,6 +169,7 @@ static void init(grib_accessor* a,const long v, grib_arguments* args)
     const char * user_lib=NULL;
     grib_accessor_data_jpeg2000_packing *self =(grib_accessor_data_jpeg2000_packing*)a;
 
+    self->jpeg_lib = 0;
     self->type_of_compression_used  = grib_arguments_get_name(a->parent->h,args,self->carg++);
     self->target_compression_ratio = grib_arguments_get_name(a->parent->h,args,self->carg++);
     self->ni                     = grib_arguments_get_name(a->parent->h,args,self->carg++);
@@ -212,7 +213,7 @@ static int value_count(grib_accessor* a,long* n_vals)
 
 #define EXTRA_BUFFER_SIZE 10240
 
-#ifdef HAVE_JPEG
+#if HAVE_JPEG
 static int  unpack_double(grib_accessor* a, double* val, size_t *len)
 {
     grib_accessor_data_jpeg2000_packing *self =(grib_accessor_data_jpeg2000_packing*)a;
@@ -286,6 +287,9 @@ static int  unpack_double(grib_accessor* a, double* val, size_t *len)
         if ((err = grib_jasper_decode(a->parent->h->context,buf,&buflen,val,&n_vals)) != GRIB_SUCCESS)
             return err;
         break;
+    default:
+        grib_context_log(a->parent->h->context,GRIB_LOG_ERROR,"Unable to unpack. Invalid JPEG library.\n");
+        return GRIB_DECODING_ERROR;
     }
 
     *len = n_vals;
@@ -527,14 +531,14 @@ static int pack_double(grib_accessor* a, const double* cval, size_t *len)
 static int  unpack_double(grib_accessor* a, double* val, size_t *len)
 {
     grib_context_log(a->parent->h->context, GRIB_LOG_ERROR,
-            "jpeg support not enabled. Please rerun configure with the --with-jasper or --with-openjpeg option.");
+            "JPEG support not enabled.");
     return GRIB_NOT_IMPLEMENTED;
 }
 
 static int pack_double(grib_accessor* a, const double* val, size_t *len)
 {
     grib_context_log(a->parent->h->context, GRIB_LOG_ERROR,
-            "jpeg support not enabled. Please rerun configure with the --with-jasper or --with-openjpeg option.");
+            "JPEG support not enabled.");
     return GRIB_NOT_IMPLEMENTED;
 }
 
